package com.zsc.edu.dify.modules.dify.controller;

import com.zsc.edu.dify.modules.dify.repo.AppEntityRepository;
import io.github.guoshiqiufeng.dify.chat.DifyChat;
import io.github.guoshiqiufeng.dify.chat.dto.request.ChatMessageSendRequest;
import io.github.guoshiqiufeng.dify.chat.dto.request.MessageConversationsRequest;
import io.github.guoshiqiufeng.dify.chat.dto.response.ChatMessageSendCompletionResponse;
import io.github.guoshiqiufeng.dify.chat.dto.response.ChatMessageSendResponse;
import io.github.guoshiqiufeng.dify.chat.dto.response.MessageConversationsResponse;
import io.github.guoshiqiufeng.dify.core.pojo.DifyPageResult;
import jakarta.annotation.Resource;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 * @author yanghq
 * @version 1.0
 * @since 2025/3/25 10:48
 */
@RestController
@RequestMapping("/api/v1/chat")
public class V1ChatController {

    @Resource
    private DifyChat difyChat;
    @Resource
    private AppEntityRepository appEntityRepository;

    /**
     * 发送消息
     *
     * @param sendRequest 消息参数 （可以自定义参数，调用 difyChat 实例时重新组装即可），
     *                    用户 id可以改为从上下文（token）获取，
     *                    apikey 建议在数据库进行存储，前端调用时传智能体 id，从数据库查询
     */
    @PostMapping("/completions/{appid}")
    @PreAuthorize("hasAuthority('dify:chat:query')")
    public ChatMessageSendResponse sendChatMessage(
            @RequestBody ChatMessageSendRequest sendRequest,
            @PathVariable String appid
    ) {
        String apiKey = appEntityRepository.selectApiKey(appid);
        sendRequest.setApiKey(apiKey);
        return difyChat.send(sendRequest);
    }


    /**
     * 发送消息（流式）
     *
     * @param sendRequest 消息参数 （可以自定义参数，调用 difyChat 实例时重新组装即可），
     *                    用户 id可以改为从上下文（token）获取，
     *                    apikey 建议在数据库进行存储，前端调用时传智能体 id，从数据库查询
     */
    @PostMapping(value = "/completions/stream/{appid}", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
//    @PreAuthorize("hasAuthority('difyChat:query')")
    public Flux<ChatMessageSendCompletionResponse> sendChatMessageStream(
            @RequestBody ChatMessageSendRequest sendRequest,
            @PathVariable String appid
    ) {
        String apiKey = appEntityRepository.selectApiKey(appid);
        sendRequest.setApiKey(apiKey);
        return difyChat.sendChatMessageStream(sendRequest);
    }

    /**
     * 获取会话列表
     *
     * @param request 请求参数
     * @return 会话列表
     */
    @PostMapping("/conversations/{appid}")
    @PreAuthorize("hasAuthority('dify:chat:query')")
    public DifyPageResult<MessageConversationsResponse> conversations(
            @RequestBody MessageConversationsRequest request,
            @PathVariable String appid
    ) {
        String apiKey = appEntityRepository.selectApiKey(appid);
        request.setApiKey(apiKey);
        return difyChat.conversations(request);
    }

    /**
     * 停止流式消息
     *
     * @param taskId  任务id
     * @param userId  用户id
     */
    @PatchMapping("/stream/stop")
    public void stopMessagesStream(@RequestParam String taskId, @RequestParam String userId, @RequestParam String appid) {
        String apiKey = appEntityRepository.selectApiKey(appid);
        difyChat.stopMessagesStream(apiKey, taskId, userId);
    }

    /**
     * 删除会话
     *
     * @param conversationId 会话id
     * @param userId         用户id
     */
    @DeleteMapping("/conversation")
    @PreAuthorize("hasAuthority('dify:chat:delete')")
    public void deleteConversation(@RequestParam String conversationId, @RequestParam String userId, @RequestParam String appid) {
        String apiKey = appEntityRepository.selectApiKey(appid);
        difyChat.deleteConversation(conversationId, apiKey, userId);
    }
}